@@ -1,5 +1,6 @@
 # %% Imports
 import pandas as pd
+pd.options.mode.copy_on_write = True
 import numpy as np
 import os
 import sys
@@ -10,13 +11,23 @@ import tlid
 
 
 from jgtutils.jgtos import get_data_path
+from mlutils import get_outfile_fullpath
+from mlconstants import MX_NS
 
-
-from jgtutils.jgtconstants import *
+#from jgtutils.jgtconstants import *
+from jgtutils.jgtconstants import (VECTOR_AO_FDBS, VECTOR_AO_FDBB, VECTOR_AO_FDBS_COUNT, VECTOR_AO_FDBB_COUNT, VECTOR_AO_FDB_COUNT,ML_DEFAULT_COLUMNS_TO_KEEP ,FDBB,FDBS,AO,ZLCB,ZLCS,OPEN,LOW,CLOSE,HIGH,DATE,
+                                   FDBB as SIGNAL_BUY_COLN,
+                                   FDBS as SIGNAL_SELL_COLN,
+                                   FDB_TARGET as __TARGET,
+                                   FDB as SIGNAL_COLN
+                                   )
 
 
 # %% Functions
 
+def set_target_variable_name(target_name):
+    global __TARGET
+    __TARGET = target_name
 
 def _crop_dataframe(df, crop_last_dt: str = None, crop_start_dt: str = None):
     if crop_last_dt is not None:
@@ -53,39 +64,39 @@ def calculate_target_variable_min_max(
     df["tmin"] = float("nan")
     df["p"] = float("nan")
     df["l"] = float("nan")
-    df["target"] = float("nan")
+    df[__TARGET] = float("nan")
 
     # Calculate the maximum and minimum Close value in the window range for each row
     for i in range(WINDOW_MIN, len(df) - WINDOW_MAX):
         # FDBS
-        if df.loc[i, "fdbs"] == 1.0:
-            df.loc[i, "tmax"] = df.loc[i : i + WINDOW_MAX, "Close"].min()
-            df.loc[i, "tmin"] = df.loc[i : i + WINDOW_MAX, "Close"].max()
+        if df.loc[i, SIGNAL_SELL_COLN] == 1.0:
+            df.loc[i, "tmax"] = df.loc[i : i + WINDOW_MAX, CLOSE].min()
+            df.loc[i, "tmin"] = df.loc[i : i + WINDOW_MAX, CLOSE].max()
 
-            if df.loc[i, "High"] < df.loc[i, "tmin"]:
-                df.loc[i, "l"] = round(df.loc[i, "High"] - df.loc[i, "Low"], rounder)
-                df.loc[i, "target"] = round(
-                    -1 * (df.loc[i, "High"] - df.loc[i, "Low"]), rounder
+            if df.loc[i, HIGH] < df.loc[i, "tmin"]:
+                df.loc[i, "l"] = round(df.loc[i, HIGH] - df.loc[i, LOW], rounder)
+                df.loc[i, __TARGET] = round(
+                    -1 * (df.loc[i, HIGH] - df.loc[i, LOW]), rounder
                 )
             else:
-                df.loc[i, "p"] = round(df.loc[i, "Low"] - df.loc[i, "tmax"], rounder)
-                df.loc[i, "target"] = round(
-                    df.loc[i, "Low"] - df.loc[i, "tmax"], rounder
+                df.loc[i, "p"] = round(df.loc[i, LOW] - df.loc[i, "tmax"], rounder)
+                df.loc[i, __TARGET] = round(
+                    df.loc[i, LOW] - df.loc[i, "tmax"], rounder
                 )
         # FDBB
-        if df.loc[i, "fdbb"] == 1.0:
-            df.loc[i, "tmax"] = df.loc[i : i + WINDOW_MAX, "Close"].max()
-            df.loc[i, "tmin"] = df.loc[i : i + WINDOW_MAX, "Close"].min()
+        if df.loc[i, SIGNAL_BUY_COLN] == 1.0:
+            df.loc[i, "tmax"] = df.loc[i : i + WINDOW_MAX, CLOSE].max()
+            df.loc[i, "tmin"] = df.loc[i : i + WINDOW_MAX, CLOSE].min()
 
-            if df.loc[i, "Low"] > df.loc[i, "tmin"]:
-                df.loc[i, "l"] = round(df.loc[i, "High"] - df.loc[i, "Low"], rounder)
-                df.loc[i, "target"] = round(
-                    -1 * (df.loc[i, "High"] - df.loc[i, "Low"]), rounder
+            if df.loc[i, LOW] > df.loc[i, "tmin"]:
+                df.loc[i, "l"] = round(df.loc[i, HIGH] - df.loc[i, LOW], rounder)
+                df.loc[i, __TARGET] = round(
+                    -1 * (df.loc[i, HIGH] - df.loc[i, LOW]), rounder
                 )
             else:
-                df.loc[i, "p"] = round(df.loc[i, "tmax"] - df.loc[i, "High"], rounder)
-                df.loc[i, "target"] = round(
-                    df.loc[i, "tmax"] - df.loc[i, "High"], rounder
+                df.loc[i, "p"] = round(df.loc[i, "tmax"] - df.loc[i, HIGH], rounder)
+                df.loc[i, __TARGET] = round(
+                    df.loc[i, "tmax"] - df.loc[i, HIGH], rounder
                 )
 
     # Fill NaN with zero for columns tmax and tmin
@@ -93,16 +104,16 @@ def calculate_target_variable_min_max(
     df["tmin"] = df["tmin"].fillna(0)
     df["p"] = df["p"].fillna(0)
     df["l"] = df["l"].fillna(0)
-    df["target"] = df["target"].fillna(0)
+    df[__TARGET] = df[__TARGET].fillna(0)
     # After calculating the 'target' column
     if pipsize != -1:
-        df["target"] = df["target"] / pipsize
+        df[__TARGET] = df[__TARGET] / pipsize
 
     # @q Maybe set backnthe index !??
     if set_index:
         #parse Date as datetime
-        df['Date'] = pd.to_datetime(df['Date'])
-        df.set_index("Date", inplace=True)
+        df[DATE] = pd.to_datetime(df[DATE])
+        df.set_index(DATE, inplace=True)
         
     return df
 
@@ -116,27 +127,38 @@ def pto_target_calculation(
     WINDOW_MIN:int=1,
     WINDOW_MAX:int=150,
     output_report_dir=None,
-    pto_vec_fdb_ao_vector_window_flag=False,
-    drop_calc_col=False,
+    pto_vec_fdb_ao_vector_window_flag=True,
+    drop_calc_col=True,
     write_reporting=True,
     calc_col_to_drop_names=["tmax", "tmin", "p", "l"],
     sel_1_suffix="_sel",
-    sel_1_keeping_columns=["High","Low", "fdbs", "fdbb", "tmax", "tmin", "p", "l", "target"],
+    sel_1_keeping_columns=[HIGH,LOW, FDBS, FDBB, "tmax", "tmin", "p", "l", __TARGET],
     sel_2_suffix="_tnd",
-    sel_2_keeping_columns=["Open", "High", "Low", "Close", "fdbs", "fdbb", "target"],
-    pto_vec_fdb_ao_out_s_name="vaos",
-    pto_vec_fdb_ao_out_b_name="vaob",
-    pto_vec_fdb_ao_in_s_sig_name="fdbs",
-    pto_vec_fdb_ao_in_s_win_end_sig_name="zlcb",
-    pto_vec_fdb_ao_in_b_sig_name="fdbb",
-    pto_vec_fdb_ao_in_b_win_end_sig_name="zlcs",
-    pto_vec_fdb_ao_in_t_val_name="ao",
+    sel_2_keeping_columns=[SIGNAL_COLN, __TARGET],
+    pto_vec_fdb_ao_out_s_name=VECTOR_AO_FDBS,#"vaos",
+    pto_vec_fdb_ao_out_b_name=VECTOR_AO_FDBB,#"vaob",
+    pto_vec_fdb_ao_in_s_sig_name=FDBS,#"fdbs",
+    pto_vec_fdb_ao_in_s_win_end_sig_name=ZLCB,#"zlcb",
+    pto_vec_fdb_ao_in_b_sig_name=FDBB,#"fdbb",
+    pto_vec_fdb_ao_in_b_win_end_sig_name=ZLCS,#"zlcs",
+    pto_vec_fdb_ao_in_t_val_name=AO,#"ao",
     additional_columns_to_drop=None,
     selected_columns_to_keep=None,
     save_outputs=True,
     only_if_target_exist_n_not_zero=True,
-    use_fresh=False,
+    use_fresh=True,
     keep_bid_ask=True,
+    regenerate_cds=False,
+    gator_oscillator_flag=False,
+    mfi_flag=True,
+    balligator_flag=False,
+    balligator_period_jaws=89,
+    largest_fractal_period=89,
+    talligator_flag=False,
+    talligator_period_jaws=377,
+    use_ttf=True,
+    pn="ttf",
+    drop_vector_ao_intermediate_array=True
 ):
     """
     Prototype Calculation of target based on the given POV parameters and output to file with report.
@@ -153,9 +175,9 @@ def pto_target_calculation(
         write_reporting (bool, optional): If True, write the reporting. Defaults to True.
         calc_col_to_drop_names (list, optional): The list of column names to drop. Defaults to ["tmax", "tmin", "p", "l"].
         sel_1_suffix (str, optional): The suffix for the first selection. Defaults to "_sel".
-        sel_1_keeping_columns (list, optional): The list of columns to keep for the first selection. Defaults to ["Low", "fdbs", "fdbb", "tmax", "tmin", "p", "l", "target"].
+        sel_1_keeping_columns (list, optional): The list of columns to keep for the first selection. Defaults to [LOW, "fdbs", "fdbb", "tmax", "tmin", "p", "l", __TARGET].
         sel_2_suffix (str, optional): The suffix for the second selection. Defaults to "_tnd".
-        sel_2_keeping_columns (list, optional): The list of columns to keep for the second selection. Defaults to ["Open", "High", "Low", "Close", "fdbs", "fdbb", "target"].  
+        sel_2_keeping_columns (list, optional): The list of columns to keep for the second selection. Defaults to ["Open", HIGH, LOW, CLOSE, "fdbs", "fdbb", __TARGET].  
         pto_vec_fdb_ao_out_s_name (str, optional): The name of the output for fdb_ao_vector_window. Defaults to "vaos".
         pto_vec_fdb_ao_out_b_name (str, optional): The name of the output for fdb_ao_vector_window. Defaults to "vaob".
         pto_vec_fdb_ao_in_s_sig_name (str, optional): The name of the input for fdb_ao_vector_window. Defaults to "fdbs".
@@ -169,6 +191,18 @@ def pto_target_calculation(
         only_if_target_exist_n_not_zero (bool, optional): If True, only if target exists and not zero. Defaults to True.
         use_fresh (bool, optional): If True, use fresh data. Defaults to False.
         keep_bid_ask (bool, optional): If True, keep bid and ask. Defaults to True.
+        regenerate_cds (bool, optional): If True, regenerate the CDS. Defaults to False.
+        gator_oscillator_flag (bool, optional): If True, calculate the Gator Oscillator. Defaults to False.
+        mfi_flag (bool, optional): If True, calculate the MFI. Defaults to False.
+        balligator_flag (bool, optional): If True, calculate the Alligator. Defaults to False.
+        balligator_period_jaws (int, optional): The period for the Alligator jaws. Defaults to 89.
+        largest_fractal_period (int, optional): The period for the largest fractal. Defaults to 89.
+        talligator_flag (bool, optional): If True, calculate the T-Alligator. Defaults to False.
+        talligator_period_jaws (int, optional): The period for the T-Alligator jaws. Defaults to 377.
+        use_ttf (bool, optional): If True, use TTF. Defaults to True.
+        pn (str, optional): The midfix for TTF. Defaults to "ttf".
+        drop_vector_ao_intermediate_array (bool, optional): If True, drop the vector AO intermediate array. Defaults to True. (we have the Counts vaosc,vaosb)
+
         
 
     Returns:
@@ -186,6 +220,9 @@ def pto_target_calculation(
     outdir_tmx = os.path.join(
         data_dir_full, "targets", "mx"
     )  # @STCIssue Hardcoded path future JGTPY_DATA_FULL/.../mx
+    #make the outdir_tmx if not exist
+    os.makedirs(outdir_tmx, exist_ok=True)
+    
     df_result_tmx, sel1, sel2 = _pov_target_calculation_n_output240223(
         indir_cds=indir_cds,
         outdir_tmx=outdir_tmx,
@@ -218,7 +255,17 @@ def pto_target_calculation(
         only_if_target_exist_n_not_zero=only_if_target_exist_n_not_zero,
         use_fresh=use_fresh,
         keep_bid_ask=keep_bid_ask,
-        
+        regenerate_cds=regenerate_cds,
+        gator_oscillator_flag=gator_oscillator_flag,
+        mfi_flag=mfi_flag,
+        balligator_flag=balligator_flag,
+        balligator_period_jaws=balligator_period_jaws,
+        largest_fractal_period=largest_fractal_period,
+        talligator_flag=talligator_flag,
+        talligator_period_jaws=talligator_period_jaws, 
+        use_ttf=use_ttf,
+        pn=pn,
+        drop_vector_ao_intermediate_array=drop_vector_ao_intermediate_array
     )
     return df_result_tmx, sel1, sel2
 
@@ -234,38 +281,47 @@ def _pov_target_calculation_n_output240223(
     WINDOW_MIN=1,
     WINDOW_MAX=150,
     output_report_dir=None,
-    pto_vec_fdb_ao_vector_window_flag=False,
+    pto_vec_fdb_ao_vector_window_flag=True,
     drop_calc_col=True,
     write_reporting=True,
     calc_col_to_drop_names=["tmax", "tmin", "p", "l"],
     sel_1_suffix="_sel",
-    sel_1_keeping_columns=[
-        "High",
-        "Low",
-        "fdbs",
-        "fdbb",
-        "tmax",
-        "tmin",
-        "p",
-        "l",
-        "target",
-    ],
+    sel_1_keeping_columns=[HIGH,
+                           LOW,                    
+                           FDBS,                   
+                           FDBB,                   
+                           "tmax",                 
+                           "tmin",
+                           "p",
+                           "l", 
+                           __TARGET],
     sel_2_suffix="_tnd",
-    sel_2_keeping_columns=["Open", "High", "Low", "Close", "fdbs", "fdbb", "target"],
-    pto_vec_fdb_ao_out_s_name="vaos",
-    pto_vec_fdb_ao_out_b_name="vaob",
-    pto_vec_fdb_ao_in_s_sig_name="fdbs",
-    pto_vec_fdb_ao_in_s_win_end_sig_name="zlcb",
-    pto_vec_fdb_ao_in_b_sig_name="fdbb",
-    pto_vec_fdb_ao_in_b_win_end_sig_name="zlcs",
-    pto_vec_fdb_ao_in_t_val_name="ao",
+    sel_2_keeping_columns=[ SIGNAL_COLN, __TARGET],
+    pto_vec_fdb_ao_out_s_name=VECTOR_AO_FDBS,
+    pto_vec_fdb_ao_out_b_name=VECTOR_AO_FDBB,
+    pto_vec_fdb_ao_in_s_sig_name=FDBS,
+    pto_vec_fdb_ao_in_s_win_end_sig_name=ZLCB,
+    pto_vec_fdb_ao_in_b_sig_name=FDBB,
+    pto_vec_fdb_ao_in_b_win_end_sig_name=ZLCS,
+    pto_vec_fdb_ao_in_t_val_name=AO,
     additional_columns_to_drop=None,
     selected_columns_to_keep=None,
     save_outputs=True,
     only_if_target_exist_n_not_zero=True,
-    use_fresh=False,
+    use_fresh=True,
     keep_fdb_count_separated_columns=False,
     keep_bid_ask=True,
+    regenerate_cds=False,
+    gator_oscillator_flag=False,
+    mfi_flag=True,
+    balligator_flag=False,
+    balligator_period_jaws=89,
+    largest_fractal_period=89,
+    talligator_flag=False,
+    talligator_period_jaws=377,
+    use_ttf=True,
+    pn="ttf",
+    drop_vector_ao_intermediate_array=True,
 ):
     if tlid_tag is None:
         tlid_tag = tlid.get_minutes()
@@ -278,6 +334,7 @@ def _pov_target_calculation_n_output240223(
         len(str(pipsize).split(".")[1]) if "." in str(pipsize) else len(str(pipsize))
     )
     rounder = nb_decimal
+    pn=pn
 
     # Possible crop of the dataframe to a specific date range
 
@@ -285,14 +342,59 @@ def _pov_target_calculation_n_output240223(
     
     #@STCIssue Generate if not exist
     cds_full_filename = f"{indir_cds}/{ifn}_{t}.csv"
-    if not os.path.exists(cds_full_filename):
+    if not os.path.exists(cds_full_filename) or regenerate_cds or use_fresh:
         from jgtpy import JGTCDS as cds
-        print("JTC is generating the CDS file from PDS file because it could not find it on disk.")
-        cds.createFromPDSFileToCDSFile(instrument=i, timeframe=t,use_full=True,use_fresh=use_fresh,keep_bid_ask=keep_bid_ask)#@STCGoal Use Fresh
+        reason = "(dont exist)" if not os.path.exists(cds_full_filename) else "" 
+        msg = f"JTC is generating the CDS file{reason}: "
+        msg = msg + " regenerated flag active" if regenerate_cds else msg
+        msg = msg + " use fresh flag active" if use_fresh else msg
+
+        print(msg)
+        print("EXITING - RUN PREREQ SCRIPTS BEFORE RUNNING THIS SCRIPT")
+        print("")
+        sys.exit(1)
+        from realityhelper import generate_mlf_feature_pattern
+        generate_mlf_feature_pattern(i, t, use_full=True, force_refresh=True, pn=pn) #@STCIssue WE DONT HAVE A GENERIC LIST OF COLUMNS BY PATTERNS TO LOAD
         
-    df_cds_source = pd.read_csv(
-        cds_full_filename, index_col=0, parse_dates=True
-    )
+        
+        
+        
+        use_full = True
+        #@STCIssue UPGRADE to use JGTCDSSvc
+        from jgtpy import JGTCDSSvc as svc
+        svc.get(instrument=i, timeframe=t, use_full=use_full, use_fresh=use_fresh)
+        # cds.createFromPDSFileToCDSFile(instrument=i, 
+        #                                timeframe=t,
+        #                                use_full=use_full,
+        #                                use_fresh=use_fresh,keep_bid_ask=keep_bid_ask,
+        #                                gator_oscillator_flag=gator_oscillator_flag,
+        #                                mfi_flag=mfi_flag,
+        #                                balligator_flag=balligator_flag,
+        #                                balligator_period_jaws=balligator_period_jaws,
+        #                                largest_fractal_period=largest_fractal_period,
+        #                                talligator_flag=talligator_flag,
+        #                                talligator_period_jaws=talligator_period_jaws)#@STCGoal Use Fresh
+    if use_ttf:
+        from ptottf import read_ttf_csv
+        from mldatahelper import read_mlf_for_pattern
+        
+        print("                                 jgtml is using ttf....")
+        #df_cds_source:pd.DataFrame = read_ttf_csv(i, t, use_full=True,pn=pn)
+        df_cds_source:pd.DataFrame = read_mlf_for_pattern(i, t, use_full=True,pn=pn)
+        #@STCGoal Pattern Name -> We have the Columns list serialized
+        from mldatahelper import read_patternname_columns_list,read_mlf_pattern_lagging_columns_list
+        laggingFeatureColumns = read_mlf_pattern_lagging_columns_list(i,t,use_full=True,pn=pn)
+        
+        sel_2_keeping_columns = sel_2_keeping_columns+laggingFeatureColumns
+        print("INFO::Lagging Columns list from higher TF:",laggingFeatureColumns)
+        print(">   We would use that to filter the columns and get our training data out of this refactored module (JTC.py)  -or shall I say from prototype to production/new module.  -> OUTPUT:  Training Data and Reality Data with which we would predict using our model.")
+        #print(f"Does these columns read from the source we would use to create the MXTarget data align with the input pattern {pn}:",df_cds_source.columns)
+        #sys.exit(0)
+        #print("JGTML::Debug len of df_cds_source:", len(df_cds_source))
+    else:
+        df_cds_source = pd.read_csv(
+            cds_full_filename, index_col=0, parse_dates=True
+        )
 
     df_result_tmx = calculate_target_variable_min_max(
         df_cds_source, 
@@ -303,6 +405,7 @@ def _pov_target_calculation_n_output240223(
         WINDOW_MIN=WINDOW_MIN,
         WINDOW_MAX=WINDOW_MAX,
     )
+    #print("calculate_target_variable_min_max::",df_result_tmx.columns)
 
     if pto_vec_fdb_ao_vector_window_flag:
         df_result_tmx = get_fdb_ao_vector_window(
@@ -317,13 +420,21 @@ def _pov_target_calculation_n_output240223(
             only_if_target_exist_n_not_zero=only_if_target_exist_n_not_zero,
             
         )
+        #print("after : get_fdb_ao_vector_window::",df_result_tmx.columns)
+        # print(df_result_tmx.tail(14))
         #@STCGoal Count of the ao vector in the window
-        df_result_tmx.loc[:, VECTOR_AO_FDBS_COUNT] = df_result_tmx[VECTOR_AO_FDBS].apply(lambda x: len(x.split(',')))
-        df_result_tmx.loc[:, VECTOR_AO_FDBB_COUNT] = df_result_tmx[VECTOR_AO_FDBB].apply(lambda x: len(x.split(',')))
-        
+        df_result_tmx.loc[:, VECTOR_AO_FDBS_COUNT] = df_result_tmx[VECTOR_AO_FDBS].apply(lambda x: len(x.split(',')) if isinstance(x, str) else 0)
+        df_result_tmx.loc[:, VECTOR_AO_FDBB_COUNT] = df_result_tmx[VECTOR_AO_FDBB].apply(lambda x: len(x.split(',')) if isinstance(x, str) else 0)
+       
         df_result_tmx.loc[:, VECTOR_AO_FDBS_COUNT] = df_result_tmx.apply(lambda x: 0 if x[VECTOR_AO_FDBS_COUNT] == 1 else x[VECTOR_AO_FDBS_COUNT], axis=1)
         df_result_tmx.loc[:, VECTOR_AO_FDBB_COUNT] = df_result_tmx.apply(lambda x: 0 if x[VECTOR_AO_FDBB_COUNT] == 1 else x[VECTOR_AO_FDBB_COUNT], axis=1)
         
+        if drop_vector_ao_intermediate_array:
+            df_result_tmx.drop(columns=[VECTOR_AO_FDBS, VECTOR_AO_FDBB], inplace=True)
+    
+        
+        #df_result_tmx.to_csv("debug.csv",index=True)
+        #sys.exit(0)
         if keep_fdb_count_separated_columns:                
             sel_1_keeping_columns.append(VECTOR_AO_FDBS_COUNT)
             sel_2_keeping_columns.append(VECTOR_AO_FDBS_COUNT)
@@ -339,28 +450,32 @@ def _pov_target_calculation_n_output240223(
         try:selected_columns_to_keep.append(VECTOR_AO_FDB_COUNT)
         except:pass
         
+        df_result_tmx.loc[:, VECTOR_AO_FDB_COUNT] = df_result_tmx[VECTOR_AO_FDBS_COUNT] + df_result_tmx[VECTOR_AO_FDBB_COUNT]
         #VECTOR_AO_FDB_COUNT column gets the value of the vector ao fdbs count or fdbb count if not 1
+        #df_result_tmx.loc[:, VECTOR_AO_FDB_COUNT] = df_result_tmx.apply(lambda x: x[VECTOR_AO_FDBS_COUNT] if x[VECTOR_AO_FDBS_COUNT] != 0 else x[VECTOR_AO_FDBB_COUNT] != 0, axis=1)
         
-        df_result_tmx.loc[:, VECTOR_AO_FDB_COUNT] = df_result_tmx.apply(lambda x: x[VECTOR_AO_FDBS_COUNT] if x[VECTOR_AO_FDBS_COUNT] != 1 else x[VECTOR_AO_FDBB_COUNT] != 1, axis=1)
+        # df_result_tmx.loc[:, VECTOR_AO_FDB_COUNT] = df_result_tmx.apply(lambda x: x[VECTOR_AO_FDBS_COUNT] if x[VECTOR_AO_FDBS_COUNT] != 1 else x[VECTOR_AO_FDBB_COUNT] != 1, axis=1)
         
-        df_result_tmx.loc[:, VECTOR_AO_FDB_COUNT] = df_result_tmx.apply(lambda x: 0 if x[VECTOR_AO_FDB_COUNT] == 1 else x[VECTOR_AO_FDB_COUNT], axis=1)
+        # df_result_tmx.loc[:, VECTOR_AO_FDB_COUNT] = df_result_tmx.apply(lambda x: 0 if x[VECTOR_AO_FDB_COUNT] == 1 else x[VECTOR_AO_FDB_COUNT], axis=1)
         
-        #fill with zero if nan or 1
+        # #fill with zero if nan or 1
         df_result_tmx[VECTOR_AO_FDB_COUNT] = df_result_tmx[VECTOR_AO_FDB_COUNT].fillna(0)
-        
+        #VECTOR_AO_FDB_COUNT column gets the value of the vector ao fdbs count or fdbb count if not 0
+        #df_result_tmx.loc[:, VECTOR_AO_FDB_COUNT] = df_result_tmx.apply(lambda x: x[VECTOR_AO_FDBS_COUNT] if x[VECTOR_AO_FDBS_COUNT] != 0 else x[VECTOR_AO_FDBB_COUNT] != 0, axis=1)
+        #df_result_tmx.loc[:, VECTOR_AO_FDB_COUNT] =
         
         #df_result_tmx[VECTOR_AO_FDB_COUNT] = df_result_tmx[VECTOR_AO_FDB_COUNT].fillna(0)
         
-        if pto_vec_fdb_ao_out_s_name  not in sel_1_keeping_columns:
+        if pto_vec_fdb_ao_out_s_name  not in sel_1_keeping_columns and not drop_vector_ao_intermediate_array:
             sel_1_keeping_columns.append(pto_vec_fdb_ao_out_s_name )
         
-        if pto_vec_fdb_ao_out_b_name not in sel_1_keeping_columns:
+        if pto_vec_fdb_ao_out_b_name not in sel_1_keeping_columns and not drop_vector_ao_intermediate_array:
             sel_1_keeping_columns.append(pto_vec_fdb_ao_out_b_name )
             
-        if pto_vec_fdb_ao_out_s_name  not in sel_2_keeping_columns:
+        if pto_vec_fdb_ao_out_s_name  not in sel_2_keeping_columns and not drop_vector_ao_intermediate_array:
             sel_2_keeping_columns.append(pto_vec_fdb_ao_out_s_name )
         
-        if pto_vec_fdb_ao_out_b_name not in sel_2_keeping_columns:
+        if pto_vec_fdb_ao_out_b_name not in sel_2_keeping_columns and not drop_vector_ao_intermediate_array:
             sel_2_keeping_columns.append(pto_vec_fdb_ao_out_b_name )      
         
         
@@ -383,36 +498,48 @@ def _pov_target_calculation_n_output240223(
 
 
     if save_outputs:
-        output_sel_cols_fn = f"{outdir_tmx}/{ifn}_{t}{sel_1_suffix}.csv"
+        print("INFO::Saving MX Target data to file...")
+        output_sel1_cols_fn = get_outfile_fullpath(i,t,use_full=True,ns=MX_NS,pn=pn,suffix=sel_1_suffix)
+        #output_sel_cols_fn = f"{outdir_tmx}/{ifn}_{t}{sel_1_suffix}.csv"
         try:
-            sel1.to_csv(output_sel_cols_fn, index=True)
-            print(f"Saved to {output_sel_cols_fn}")
+            sel1.to_csv(output_sel1_cols_fn, index=True)
+            print(f"INFO::Saved to {output_sel1_cols_fn}")
         except Exception as e:
-            print(f"Error occurred while saving to {output_sel_cols_fn}: {str(e)}")
+            print(f"Error occurred while saving to {output_sel1_cols_fn}: {str(e)}")
     
     sel2 = df_result_tmx[sel_2_keeping_columns].copy()
-    sel2["target"] = sel2["target"].round(rounder)
-    sel2 = sel2[(sel2["target"] != 0)]
+    sel2[__TARGET] = sel2[__TARGET].round(rounder)
+    sel2 = sel2[(sel2[__TARGET] != 0)]
 
-    output_tnd_targetNdata_fn = f"{outdir_tmx}/{ifn}_{t}{sel_2_suffix}.csv"
+    output_tnd_sel2_targetNdata_fn =get_outfile_fullpath(i,t,use_full=True,ns=MX_NS,pn=pn,suffix=sel_2_suffix)
+    #output_tnd_targetNdata_fn = f"{outdir_tmx}/{ifn}_{t}{sel_2_suffix}.csv"
     
     if save_outputs:
         try:
-            sel2.to_csv(output_tnd_targetNdata_fn, index=True)
-            print(f"Saved to {output_tnd_targetNdata_fn}")
+            sel2.to_csv(output_tnd_sel2_targetNdata_fn, index=True)
+            print(f"INFO::Saved to {output_tnd_sel2_targetNdata_fn}")
         except Exception as e:
-            print(f"Error occurred while saving to {output_tnd_targetNdata_fn}: {str(e)}")
+            print(f"ERROR::occurred while saving to {output_tnd_sel2_targetNdata_fn}: {str(e)}")
 
-    if drop_calc_col and selected_columns_to_keep is None:
-        df_result_tmx.drop(columns=calc_col_to_drop_names, inplace=True)
-    
+    if selected_columns_to_keep is not None:
+        print("INFO::   Selected columns to keep:", selected_columns_to_keep)
+
+    # if drop_calc_col and selected_columns_to_keep is None:
+    #     print("INFO::   Dropping calculated columns:", calc_col_to_drop_names)
+    #     df_result_tmx.drop(columns=calc_col_to_drop_names, inplace=True)
+    for col in calc_col_to_drop_names:
+        if col in df_result_tmx.columns:
+            df_result_tmx.drop(columns=[col], inplace=True)
+    #print("calc_col_to_drop_names::",calc_col_to_drop_names)
+    #print("selected_columns_to_keep::",selected_columns_to_keep)
     
         
     if additional_columns_to_drop is not None and selected_columns_to_keep is not None:
-        print("You cannot use both additional_columns_to_drop and selected_columns_to_keep. Please use only one of them.")
+        print("ERROR::  You cannot use both additional_columns_to_drop and selected_columns_to_keep. Please use only one of them.")
         return df_result_tmx, sel1, sel2
     
     if additional_columns_to_drop is not None:
+        #print("Dropping additional columns:", additional_columns_to_drop)
         try:
             df_result_tmx.drop(columns=additional_columns_to_drop, inplace=True)
         except:
@@ -426,12 +553,13 @@ def _pov_target_calculation_n_output240223(
     
     if save_outputs:
         # Save the result to a csv file
-        output_all_cols_fn = f"{outdir_tmx}/{ifn}_{t}.csv"
+        output_all_cols_fn = get_outfile_fullpath(i,t,use_full=True,ns=MX_NS,pn=pn)
+        #output_all_cols_fn = f"{outdir_tmx}/{ifn}_{t}.csv"
         try:
             df_result_tmx.to_csv(output_all_cols_fn, index=True)
-            print(f"Saved to {output_all_cols_fn}")
+            print(f"INFO::Saved to {output_all_cols_fn}")
         except Exception as e:
-            print(f"Error occurred while saving to {output_all_cols_fn}: {str(e)}")
+            print(f"ERROR::Error occurred while saving to {output_all_cols_fn}: {str(e)}")
 
     if write_reporting:
         _reporting(sel2, ifn, t, pipsize, tlid_tag, output_report_dir=output_report_dir)
@@ -447,13 +575,14 @@ def _reporting(df_selection2, ifn, t, pipsize, tlid_tag, output_report_dir=None)
         os.makedirs(output_report_dir, exist_ok=True)
 
     report_file = f"{output_report_dir}/report-calc-{tlid_tag}.txt"
-    print("Reporting to:", report_file)
-    print(" tail -f ", report_file)
-
-    with open(report_file, "a") as f:
-        f.write(f"--- {ifn}_{t} --pipsize:{pipsize}---\n")
-        f.write(f"Sum of target: {df_selection2['target'].sum()}\n\n")
-        # f.write(f" (rounded): {(df_selection2['target'].sum()).round(2)}\n\n")
+    reporting_flag=True if os.getenv("JGT_REPORTING_FLAG") == "True" else False
+    if reporting_flag:
+        print("INFO::Reporting to:", report_file)
+        print(" tail -f ", report_file)
+        with open(report_file, "a") as f:
+            f.write(f"--- {ifn}_{t} --pipsize:{pipsize}---\n")
+            f.write(f"Sum of target: {df_selection2['target'].sum()}\n\n")
+            # f.write(f" (rounded): {(df_selection2['target'].sum()).round(2)}\n\n")
 
 
 def readMXFile(
@@ -468,6 +597,9 @@ def readMXFile(
     sel_2_suffix="_tnd",
     also_read_selections=False,
     generate_if_not_exist=True,
+    dropna=True,
+    mx_targets_sub_path = "targets/mx",
+    pn="ttf",
 ):
     """
     Read a MX Target file and return a pandas DataFrame.
@@ -484,6 +616,9 @@ def readMXFile(
     sel_2_suffix (str, optional): The suffix for the second selection. Defaults to "_tnd".
     also_read_selections (bool, optional): If True, also read the selections. Defaults to False.
     generate_if_not_exist (bool, optional): If True, generate the MX Target data if it does not exist. Default is True.
+    dropna (bool, optional): If True, drop the NaN values. Default is True.
+    mx_targets_sub_path (str, optional): The sub-path for the MX Targets. Default is "targets/mx".
+    pn (str, optional): The midfix for the file name. Default is "".  We might use variation of the TTF (peaks,aoac, or other and the MX Will be saved with that midfix)
 
     Returns:
     pandas.DataFrame: The DataFrame containing the MX Target data.
@@ -491,29 +626,34 @@ def readMXFile(
     tuple: A tuple containing the DataFrame and the selections DataFrames.
     """
     # Define the file path based on the environment variable or local path
-    data_path_cds = get_data_path("targets/mx", use_full=use_full)
-    fpath = pds.mk_fullpath(instrument, timeframe, "csv", data_path_cds)
+    fpath=get_outfile_fullpath(instrument, timeframe,use_full=use_full,pn=pn,ns=mx_targets_sub_path)
+    #data_path_cds = get_data_path(mx_targets_sub_path, use_full=use_full,pn=pn)
+    #fpath = pds.mk_fullpath(instrument, timeframe, "csv", data_path_cds)
     
     try:
         mdf = pd.read_csv(fpath)
     except:
-        print(f"Error reading file {fpath}")
-        print("GENERATING THE MX Targets")
+        print(f"WARN::Error reading file {fpath}")
+        print("INFO::  GENERATING THE MX Targets")
         try:
             pto_target_calculation(instrument,timeframe,pto_vec_fdb_ao_vector_window_flag=True,
                 drop_calc_col=False,
-                selected_columns_to_keep=ML_DEFAULT_COLUMNS_TO_KEEP)
+                selected_columns_to_keep=ML_DEFAULT_COLUMNS_TO_KEEP,
+                pn=pn)
         except:
-            raise ValueError(f"Error generating file {fpath}")
+            raise ValueError(f"ERROR:: generating file {fpath}")
         try:
             mdf = pd.read_csv(fpath)
         except:
             raise ValueError(f"Error reading file {fpath}")
     
 
-    # Set 'Date' as the index and convert it to datetime
-    mdf["Date"] = pd.to_datetime(mdf["Date"])
-    mdf.set_index("Date", inplace=True)
+    # Set DATE as the index and convert it to datetime
+    mdf[DATE] = pd.to_datetime(mdf[DATE])
+    mdf.set_index(DATE, inplace=True)
+
+    if dropna:
+        mdf.dropna(inplace=True)
     # Remove the specified columns
     if columns_to_remove is not None:
         mdf = mdf.drop(columns=columns_to_remove, errors="ignore")
@@ -524,11 +664,16 @@ def readMXFile(
         mdf = mdf[-quote_count:]
 
     if also_read_selections:
+        output_sel1_cols_fn = get_outfile_fullpath(instrument,timeframe,use_full=True,ns=MX_NS,pn=pn,suffix=sel_1_suffix)
         sel1 = pd.read_csv(
-            fpath.replace(".csv", f"{sel_1_suffix}.csv"), index_col=0, parse_dates=True
+            output_sel1_cols_fn,
+            index_col=0,
+            parse_dates=True
         )
+        output_tnd_sel2_targetNdata_fn =get_outfile_fullpath(instrument,timeframe,use_full=True,ns=MX_NS,pn=pn,suffix=sel_2_suffix)
         sel2 = pd.read_csv(
-            fpath.replace(".csv", f"{sel_2_suffix}.csv"), index_col=0, parse_dates=True
+            output_tnd_sel2_targetNdata_fn,
+                           index_col=0, parse_dates=True
         )
         return mdf, sel1, sel2
     return mdf
@@ -538,13 +683,13 @@ def readMXFile(
 
 def get_fdb_ao_vector_window(
     df,
-    out_s_name="vaos",#VECTOR_AO_FDBS_COUNT
-    out_b_name="vaob",#VECTOR_AO_FDBB
-    in_s_sig_name="fdbs",
-    in_s_win_end_sig_name="zlcb",
-    in_b_sig_name="fdbb",
-    in_b_win_end_sig_name="zlcs",
-    in_t_val_name="ao",
+    out_s_name=VECTOR_AO_FDBS,#"vaos",#VECTOR_AO_FDBS_COUNT
+    out_b_name=VECTOR_AO_FDBB,#"vaob",#VECTOR_AO_FDBB
+    in_s_sig_name=FDBS,#"fdbs",
+    in_s_win_end_sig_name=ZLCB,#"zlcb",
+    in_b_sig_name=FDBB,#"fdbb",
+    in_b_win_end_sig_name=ZLCS,#"zlcs",
+    in_t_val_name=AO,#"ao",
     only_if_target_exist_n_not_zero=True,
 ):
 
@@ -557,13 +702,13 @@ def get_fdb_ao_vector_window(
     df[out_s_name] = np.nan
     df[out_b_name] = np.nan
 
-    col_target_exist_in_dataset = "target" in df.columns
+    col_target_exist_in_dataset = __TARGET in df.columns
     
     
     for index, row in df.iterrows():
         process_row = True
         if only_if_target_exist_n_not_zero:
-            col_target_exist_in_dataset and row["target"] != 0
+            col_target_exist_in_dataset and row[__TARGET] != 0
             
         if row[in_s_sig_name] == 1 and row[in_t_val_name] > 0 and process_row:
             window_start = index
@@ -594,14 +739,14 @@ def get_fdb_ao_vector_window(
             df.at[index, out_b_name] = str(window.astype(float).tolist())
     # restore index
     try:
-        df.set_index("Date", inplace=True)
+        df.set_index(DATE, inplace=True)
     except:
         pass
 
     # for nan values, fill with empty string corresponding to an empty array
-    df[out_s_name].fillna("[]", inplace=True)
-    df[out_b_name].fillna("[]",inplace=True)
-
+    #df[out_s_name].fillna("[]", inplace=True)
+    #df[out_b_name].fillna("[]",inplace=True)
+    df.fillna({out_s_name: "[]", out_b_name: "[]"}, inplace=True)
     return df
 
 
@@ -633,13 +778,13 @@ def get_fdb_ao_vector_window_v2(
     df[out_s_name] = np.nan
     df[out_b_name] = np.nan
 
-    col_target_exist_in_dataset = "target" in df.columns
+    col_target_exist_in_dataset = __TARGET in df.columns
     
     
     for index, row in df.iterrows():
         process_row = True
         if only_if_target_exist_n_not_zero:
-            col_target_exist_in_dataset and row["target"] != 0
+            col_target_exist_in_dataset and row[__TARGET] != 0
             
         if row[in_s_sig_name] == 1 and row[in_t_val_name] > 0 and process_row:
             window_start = index
@@ -670,7 +815,7 @@ def get_fdb_ao_vector_window_v2(
             df.at[index, out_b_name] = str(window.astype(float).tolist())
     # restore index
     try:
-        df.set_index("Date", inplace=True)
+        df.set_index(DATE, inplace=True)
     except:
         pass
 
